/*====================================+=====================================+
! File CChannel.h                     ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/filename.h>
#include "matelefree.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CVlcMedia ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
struct str_stream
{
   /*----------------------------------------------------------------------*/
   str_stream()
   {  ; }
   str_stream( const wxString &s_name )
             : m_s_name( s_name )
   {  ; }
   str_stream( const wxString &s_name, const wxString &s_url )
             : m_s_name( s_name ), m_s_url( s_url )
   {  ; }
   /*----------------------------------------------------------------------*/
   wxString m_s_name             ; // "HD", "auto", "standard" etc.
   wxString m_s_url              ; // rtsp:// ...
   int      m_i_audio_track = -1 ; // audio track number if >= 0
   mtf::t_vec_string m_vs_option ; // m3u's "#EXTVLCOPT"

   /*----------------------------------------------------------------------*/
   static wxArrayString m_st_as_quality ;
   /*----------------------------------------------------------------------*/
   wxString get_vlc_url() const ;
   /*----------------------------------------------------------------------*/
   friend bool operator==( const str_stream &s1, const wxString &s_name )
   {  return( s1.m_s_name == s_name ) ; }
   /*----------------------------------------------------------------------*/
   static void st_init() ;
   static int st_get_quality_ind( const wxString &s )
   {  return( m_st_as_quality.Index( s ) ) ; }
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
using t_vec_stream     = std::vector< str_stream >    ;
using t_vec_stream_it  = t_vec_stream::iterator       ;
using t_vec_stream_cit = t_vec_stream::const_iterator ;

/*-------------------------------------------------------------------------*/
class CChannel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannel()
      {  ; }
      CChannel( const wxString &s_pl, const wxString &s_name,
                int i_num = -1
              ) ;
      CChannel( e_pl_known pl, const wxString &s_name,
                int i_num = -1
              ) ;
      virtual ~CChannel()
      {  ; }

      /*-------------------------------------------------------------------*/
      static constexpr const char *m_st_co_p_c_cfg_path = "/channel" ;
      static constexpr const char *m_st_co_p_c_std_french_tv_source = "frtv";

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      wxString m_s_short_name ; // name without source : France 5
      wxString m_s_name       ; // name with source / origin
      wxString m_s_name_find  ; // Prepared name for search
      /*-------------------------------------------------------------------*/
      wxString m_s_logo_fn     ; // Logo file name
      int      m_i_il_pos = -1 ; // Logo pos in image list :
                                 // -1 => Invalid, 0 => Not found
      bool     m_boo_logo_ok = false ;
      /*--( Channel info has to be saved in the config file ? )------------*/
      bool     m_boo_save_cfg = false ;

   /*----------------------------------------------------------------------*/
   protected :
      /*--( Last time the number of viewers has been fetched )-------------*/
      time_t m_ti_nb_viewers = 0 ;
      long   m_l_nb_viewers  = 0 ;

   /*----------------------------------------------------------------------*/
   public :
      /*--( Origin of this channel : fbtv, m3u )---------------------------*/
      wxString     m_s_pl_source ; // fbtv, entv etc.
      e_pl_known   m_pl_known = PK_NB ;
      /*-------------------------------------------------------------------*/
      int          m_i_num = -1 ; // Channel number : "5", "24"
      bool         m_boo_fbtvo = false ;
      t_vec_stream m_vec_stream ;
      /*--( Default track : -1 is used by VLC for "deactivated" )----------*/
      enum { DEFAULT_TRACK = -2 } ;
      /*--------------------------------------------------------------------+
      ! Why use a string instead of a vector index ?                        !
      ! This is mainly useful for channels for which we don't have a fixed  !
      ! list of streams : fb, live                                          !
      +--------------------------------------------------------------------*/
      wxString m_s_stream_name ; // Active stream name
      /*-------------------------------------------------------------------*/
      int m_i_video_track = DEFAULT_TRACK ;
      int m_i_audio_track = DEFAULT_TRACK ;
      int m_i_video_spu   = DEFAULT_TRACK ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual bool has_retry() const { return( false ) ; }
      virtual bool has_nb_viewers() const { return( false ) ; }
      virtual int get_vlcver_min() const ;
      void set_save_cfg( bool boo_val ) { m_boo_save_cfg = boo_val ; }
      bool get_save_cfg() const { return( m_boo_save_cfg ) ; }
      bool is_stream_ok() const
      {  return( !m_s_stream_name.empty() ) ; }
      int find_stream( const wxString &s_name ) const ;
      const str_stream *get_stream() ;
      wxString get_stream_name() const { return( m_s_stream_name ) ; }
      void init_fixed_streams() ;
      wxString get_short_name() const { return( m_s_short_name ) ; }
      wxString get_name() const { return( m_s_name ) ; }
      wxString get_name_find() const { return( m_s_name_find ) ; }
      wxString build_name( const wxString &s_short_name ) const ;
      void set_short_name( const wxString &s_name ) ;
      e_pl_known get_pl_known() const { return( m_pl_known ) ; }
      wxString get_pl_source() const { return( m_s_pl_source ) ; }
      void set_pl_source( const wxString &s_pl_source ) ;
      int get_num() const { return( m_i_num ) ; }
      bool is_num_set() const { return( m_i_num > 0 ) ; }
      bool is_fbtvo() const { return( m_boo_fbtvo ) ; }
      void set_fbtvo( bool boo_val ) { m_boo_fbtvo = boo_val ; }
      wxString get_pl_text_short() const ;
      wxString get_pl_text_long() const ;
      wxString get_title() ;
      void init_logo_fn() ;
      void draw_en_option_overlay( wxBitmap &bmp ) ;
      wxBitmap build_logo() ;
      wxBitmap load_logo() ;
      void init_logo() ;
      int get_logo_il_pos() ;
      wxBitmap get_logo() ;
      wxString datas_to_string() ;
      void string_to_datas( const wxString &s_val ) ;
      void rename( const wxString &s_new_name ) ;
      virtual wxString get_url() const { return( wxEmptyString ) ; }
      virtual wxString get_vlc_init() const ;
      int add_stream( str_stream &stream ) ;
      /*-------------------------------------------------------------------*/
      void reset_nb_viewers() ;
      long get_nb_viewers() const ;
      bool is_nb_viewers_refresh_needed() const ;
      virtual int refresh_nb_viewers() { return( -1 ) ; }
      /*-------------------------------------------------------------------*/
      virtual bool is_gs_retry_later() const { return( false ) ; }
      virtual void csec_streams_enter() { ; }
      virtual void csec_streams_leave() { ; }
      void message_get_all_streams( CVlcMedia *p_media = nullptr ) ;
      virtual bool is_gs_no_error() const
      {  return( true ) ; }
      virtual wxString get_gs_error_message() const
      {  return( wxEmptyString ) ; }
      virtual bool is_refresh_streams_needed() const
      {  return( false ) ; }
      virtual void reset_streams() { ; }
      virtual int get_all_streams( CVlcMedia *p_media = nullptr )
      {  return( 0 ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelDynamic : public CChannel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelDynamic( const wxString &s_pl, const wxString &s_id,
                       const wxString &s_name
                     )
                     : CChannel( s_pl, s_name ), m_s_id( s_id )
      {  ; }
      CChannelDynamic( e_pl_known pl, const wxString &s_id,
                       const wxString &s_name
                     )
                     : CChannel( pl, s_name ), m_s_id( s_id )
      {  ; }
      virtual ~CChannelDynamic()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*--( Get streams error code )---------------------------------------*/
      enum e_gs_error
      {  GSERR_NO_ERROR, GSERR_DOWNLOAD, GSERR_UNKNOWN_CHANNEL,
         GSERR_PARSING, GSERR_RETRY_LATER, GSERR_FORBIDDEN,
         GSERR_PASSWORD_PROTECTED, GSERR_NOT_VIEWABLE_HERE, GSERR_UNHANDLED,
         GSERR_NB
      } ;
      e_gs_error m_gs_error = GSERR_NO_ERROR ;
      /*--( When did the last get streams occur ? )------------------------*/
      time_t            m_ti_last_gs = 0 ;
      /*-------------------------------------------------------------------*/
      wxString          m_s_id ;
      wxCriticalSection m_csec_streams ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxString get_id() const { return( m_s_id ) ; }
      void set_id( const wxString &s_id ) { m_s_id = s_id ; }
      /*-------------------------------------------------------------------*/
      e_gs_error get_gs_error() const { return( m_gs_error ) ; }
      void set_gs_error( e_gs_error gs_error )
      {  m_gs_error = gs_error ; }
      virtual void csec_streams_enter() override
      {  m_csec_streams.Enter() ; }
      virtual void csec_streams_leave() override
      {  m_csec_streams.Leave() ; }
      virtual bool is_gs_no_error() const override
      {  return( m_gs_error == GSERR_NO_ERROR ) ; }
      virtual bool is_gs_retry_later() const override
      {  return( m_gs_error == GSERR_RETRY_LATER ) ; }
      static wxString st_get_gs_error_message( e_gs_error gse ) ;
      virtual wxString get_gs_error_message() const override
      {  return( st_get_gs_error_message( m_gs_error ) ) ; }
      /*-------------------------------------------------------------------*/
      virtual bool is_refresh_streams_needed() const override ;
      virtual void reset_streams() override ;
      int hls_extract_streams( const wxString &s_url_m3u ) ;
      void add_hls_stream( const wxString &s_url_m3u ) ;
      /*-------------------------------------------------------------------*/
      virtual int get_all_streams( CVlcMedia *p_media = nullptr ) override ;
      virtual int lookup_streams() = 0 ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelLive : public CChannelDynamic
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelLive( const wxString &s_pl, const wxString &s_id,
                    const wxString &s_name
                  )
                  : CChannelDynamic( s_pl, s_id, s_name )
      {  ; }
      CChannelLive( e_pl_known pl, const wxString &s_id,
                    const wxString &s_name
                  )
                  : CChannelDynamic( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelLive()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static constexpr const char *m_st_co_p_c_id_example = "CHANNEL_ID" ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual bool has_retry() const override { return( true ) ; }
      virtual bool has_nb_viewers() const override { return( true ) ; }
      virtual wxString get_url_format() const = 0 ;
      virtual wxString get_url_example() const = 0 ;
      virtual wxString get_url() const override
      {  return( wxString::Format( get_url_format(), m_s_id ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelDailymotion : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelDailymotion( const wxString &s_id, const wxString &s_name )
                         : CChannelLive( PK_LIVE_DM, s_id, s_name )
      {  ; }
      CChannelDailymotion( const wxString &s_pl, const wxString &s_id,
                           const wxString &s_name
                         )
                         : CChannelLive( s_pl, s_id, s_name )
      {  ; }
      CChannelDailymotion( e_pl_known pl, const wxString &s_id,
                           const wxString &s_name
                         )
                         : CChannelLive( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelDailymotion()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_pl_source() { return( "dailymotion" ) ; }
      /*-------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "https://www.dailymotion.com/video/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int lookup_streams() ;
      virtual int refresh_nb_viewers() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelTwitch : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelTwitch( const wxString &s_id, const wxString &s_name )
                    : CChannelLive( PK_LIVE_TW, s_id, s_name )
      {  ; }
      CChannelTwitch( const wxString &s_pl, const wxString &s_id,
                      const wxString &s_name
                    )
                    : CChannelLive( s_pl, s_id, s_name )
      {  ; }
      CChannelTwitch( e_pl_known pl, const wxString &s_id,
                      const wxString &s_name
                    )
                    : CChannelLive( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelTwitch()
      {  ; }

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      wxString url_encode( const wxString &s ) ;
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_pl_source() { return( "twitch" ) ; }
      /*-------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "https://www.twitch.tv/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      static e_gs_error st_get_token_sig( const wxString &s_id,
                                          wxString *p_s_token,
                                          wxString *p_s_sig
                                        ) ;
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      static mtf::t_lst_string st_build_header_client() ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int lookup_streams() override ;
      virtual int refresh_nb_viewers() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelYoutube : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelYoutube( const wxString &s_id, const wxString &s_name )
                     : CChannelLive( PK_LIVE_YT, s_id, s_name )
      {  ; }
      CChannelYoutube( const wxString &s_pl, const wxString &s_id,
                       const wxString &s_name
                     )
                     : CChannelLive( s_pl, s_id, s_name )
      {  ; }
      CChannelYoutube( e_pl_known pl, const wxString &s_id,
                       const wxString &s_name
                     )
                     : CChannelLive( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelYoutube()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*--( A channel can have multiples video live streams )--------------*/
      wxString m_s_video_id ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_pl_source() { return( "youtube" ) ; }
      /*-------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "https://www.youtube.com/channel/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int lookup_streams() override ;
      virtual int refresh_nb_viewers() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelUstream : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelUstream( const wxString &s_id, const wxString &s_name )
                     : CChannelLive( PK_LIVE_US, s_id, s_name )
      {  ; }
      CChannelUstream( const wxString &s_pl, const wxString &s_id,
                       const wxString &s_name
                     )
                     : CChannelLive( s_pl, s_id, s_name )
      {  ; }
      CChannelUstream( e_pl_known pl, const wxString &s_id,
                       const wxString &s_name
                     )
                     : CChannelLive( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelUstream()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxString m_s_username ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_pl_source() { return( "ustream" ) ; }
      /*-------------------------------------------------------------------*/
      virtual int get_vlcver_min() const override { return( 3 ) ; }
      /*--------------------------------------------------------------------+
      ! As :                                                                !
      ! 1) for Ustream "get_url()" is only used to "open the browser"       !
      ! 2) only the video id is known (and not the channel id)              !
      ! 3) I don't know how to convert the video id to the channel one      !
      ! Then : the browser URL is not the "channel" page but the embedded   !
      ! video page                                                          !
      +--------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "https://www.ustream.tv/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( "USERNAME" ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      virtual wxString get_url() const override
      {  return( st_build_url( m_s_username ) ) ; }
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int lookup_streams() override ;
      virtual int refresh_nb_viewers() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelPeriscope : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelPeriscope( const wxString &s_id, const wxString &s_name )
                       : CChannelLive( PK_LIVE_PE, s_id, s_name )
      {  ; }
      CChannelPeriscope( const wxString &s_pl, const wxString &s_id,
                         const wxString &s_name
                       )
                       : CChannelLive( s_pl, s_id, s_name )
      {  ; }
      CChannelPeriscope( e_pl_known pl, const wxString &s_id,
                         const wxString &s_name
                       )
                       : CChannelLive( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelPeriscope()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*--( The IDs will simplify the access to the number of viewers )----*/
      wxString m_s_user_id    ;
      wxString m_s_session_id ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_pl_source() { return( "periscope" ) ; }
      /*-------------------------------------------------------------------*/
      virtual int get_vlcver_min() const override { return( 3 ) ; }
      /*-------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "https://www.periscope.tv/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( "USERNAME" ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      virtual wxString get_url() const override
      {  return( st_build_url( m_s_id ) ) ; }
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int lookup_streams() override ;
      virtual int refresh_nb_viewers() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelMixer : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelMixer( const wxString &s_id, const wxString &s_name )
                   : CChannelLive( PK_LIVE_MX, s_id, s_name )
      {  ; }
      CChannelMixer( const wxString &s_pl, const wxString &s_id,
                     const wxString &s_name
                   )
                   : CChannelLive( s_pl, s_id, s_name )
      {  ; }
      CChannelMixer( e_pl_known pl, const wxString &s_id,
                     const wxString &s_name
                   )
                   : CChannelLive( pl, s_id, s_name )
      {  ; }
      virtual ~CChannelMixer()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*--( The ID will simplify the access to the number of viewers )-----*/
      wxString m_s_channel_id ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_pl_source() { return( "mixer" ) ; }
      /*-------------------------------------------------------------------*/
      virtual int get_vlcver_min() const override { return( 3 ) ; }
      /*-------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "https://mixer.com/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      virtual wxString get_url() const override
      {  return( st_build_url( m_s_id ) ) ; }
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int lookup_streams() override ;
      virtual int refresh_nb_viewers() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelDynamicKnown : public CChannelDynamic
{
   /*----------------------------------------------------------------------*/
   public :
      /*--( Channel Dynamic Type Known )-----------------------------------*/
      enum e_cdtk
      {  CDTK_M3U, CDTK_FTV, CDTK_MYTF1, CDTK_GULLI, CDTK_MELODY, CDTK_BFM,
         CDTK_NB
      } ;
      /*-------------------------------------------------------------------*/
      CChannelDynamicKnown( const wxString &s_pl, const wxString &s_name,
                            const wxString &s_url_m3u
                          )
                          : CChannelDynamic( s_pl, wxEmptyString, s_name ),
                            m_s_url_m3u( s_url_m3u )
      {  init_fnc_lookup_streams( CDTK_M3U ) ; }
      CChannelDynamicKnown( e_cdtk cdtk, const wxString &s_pl,
                            const wxString &s_id, const wxString &s_name
                          )
                          : CChannelDynamic( s_pl, s_id, s_name )
      {  init_fnc_lookup_streams( cdtk ) ; }
      CChannelDynamicKnown( e_cdtk cdtk, e_pl_known pl, const wxString &s_id,
                            const wxString &s_name
                          )
                          : CChannelDynamic( pl, s_id, s_name )
      {  init_fnc_lookup_streams( cdtk ) ; }
      virtual ~CChannelDynamicKnown()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxString m_s_url_m3u ;
      int ( CChannelDynamicKnown::*m_p_fnc_lookup_streams )() = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void set_url_m3u( const wxString &s_url ) { m_s_url_m3u = s_url ; }
      /*-------------------------------------------------------------------*/
      void init_fnc_lookup_streams( e_cdtk cdtk ) ;
      /*-------------------------------------------------------------------*/
      int lookup_streams_m3u() ;
      int lookup_streams_ftv() ;
      int lookup_streams_mytf1() ;
      int lookup_streams_gulli() ;
      int lookup_streams_melody() ;
      int lookup_streams_bfm() ;
      /*-------------------------------------------------------------------*/
      virtual int lookup_streams() override
      {  return( ( this->*m_p_fnc_lookup_streams )() ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
using t_map_channel         = std::map< wxString, CChannel * > ;
using t_map_channel_it      = t_map_channel::iterator ;
using t_map_channel_cit     = t_map_channel::const_iterator ;
/*-------------------------------------------------------------------------*/
using t_map_channel_num     = std::map< int, CChannel * > ;
using t_map_channel_num_it  = t_map_channel_num::iterator ;
using t_map_channel_num_cit = t_map_channel_num::const_iterator ;
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file CChannel.h                           !
+==========================================================================*/
