/*====================================+=====================================+
! File CVlcInterface.h                ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <vlc/vlc.h>
#include <wx/dynlib.h>
/*-------------------------------------------------------------------------*/
#include "CApp.h"
#include "matelefree.h"
/*--( Used to get version of an old libvlc releases )----------------------*/
const char *VLC_Version( void ) ;
/*-------------------------------------------------------------------------*/
#define DECL_P_FNC( f ) decltype( f ) *m_##f = nullptr
#define INIT_P_FNC( f ) m_##f = ( decltype( &f ) )GetSymbol( #f )
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CVlcInterface : public wxDynamicLibrary
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      explicit CVlcInterface( const wxString &s_vlc_path,
                              const CChannel *p_channel
                            ) ;
      ~CVlcInterface() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      libvlc_instance_t *m_inst = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static const int m_st_co_i_version_min ;
      int m_i_version = -1 ;
      /*-------------------------------------------------------------------*/
      DECL_P_FNC( libvlc_new ) ;
      DECL_P_FNC( VLC_Version ) ;
      DECL_P_FNC( libvlc_get_version ) ;
      DECL_P_FNC( libvlc_set_user_agent ) ;
      DECL_P_FNC( libvlc_set_app_id ) ;
      DECL_P_FNC( libvlc_media_player_new ) ;
      DECL_P_FNC( libvlc_media_player_event_manager ) ;
      DECL_P_FNC( libvlc_media_event_manager ) ;
      DECL_P_FNC( libvlc_media_add_option ) ;
      DECL_P_FNC( libvlc_event_attach ) ;
      DECL_P_FNC( libvlc_event_detach ) ;
      DECL_P_FNC( libvlc_event_type_name ) ;
      DECL_P_FNC( libvlc_media_player_release ) ;
      DECL_P_FNC( libvlc_release ) ;
      DECL_P_FNC( libvlc_media_player_get_xwindow ) ;
      DECL_P_FNC( libvlc_media_player_set_xwindow ) ;
      DECL_P_FNC( libvlc_media_player_get_hwnd ) ;
      DECL_P_FNC( libvlc_media_player_set_hwnd ) ;
      DECL_P_FNC( libvlc_media_new_location ) ;
      DECL_P_FNC( libvlc_media_new_path ) ;
      DECL_P_FNC( libvlc_media_player_get_media ) ;
      DECL_P_FNC( libvlc_media_player_set_media ) ;
      DECL_P_FNC( libvlc_media_get_mrl ) ;
      DECL_P_FNC( libvlc_video_get_track_description ) ;
      DECL_P_FNC( libvlc_video_get_track_count ) ;
      DECL_P_FNC( libvlc_video_get_track ) ;
      DECL_P_FNC( libvlc_video_set_track ) ;
      DECL_P_FNC( libvlc_audio_get_track_description ) ;
      DECL_P_FNC( libvlc_audio_get_track_count ) ;
      DECL_P_FNC( libvlc_audio_get_track ) ;
      DECL_P_FNC( libvlc_audio_set_track ) ;
      DECL_P_FNC( libvlc_video_get_spu_description ) ;
      DECL_P_FNC( libvlc_video_get_spu_count ) ;
      DECL_P_FNC( libvlc_track_description_list_release ) ;
      DECL_P_FNC( libvlc_video_get_spu ) ;
      DECL_P_FNC( libvlc_video_set_spu ) ;

      DECL_P_FNC( libvlc_media_release ) ;
      DECL_P_FNC( libvlc_media_player_play ) ;
      DECL_P_FNC( libvlc_media_player_pause ) ;
      DECL_P_FNC( libvlc_media_player_stop ) ;
      DECL_P_FNC( libvlc_media_player_is_playing ) ;
      DECL_P_FNC( libvlc_media_player_get_state ) ;
      DECL_P_FNC( libvlc_media_get_stats ) ;
      DECL_P_FNC( libvlc_errmsg ) ;
      DECL_P_FNC( libvlc_audio_get_volume ) ;
      DECL_P_FNC( libvlc_audio_set_volume ) ;
      DECL_P_FNC( libvlc_audio_get_mute ) ;
      DECL_P_FNC( libvlc_audio_set_mute ) ;
      DECL_P_FNC( libvlc_get_fullscreen ) ;
      DECL_P_FNC( libvlc_set_fullscreen ) ;
      DECL_P_FNC( libvlc_video_take_snapshot ) ;
      DECL_P_FNC( libvlc_video_set_mouse_input ) ;
      DECL_P_FNC( libvlc_video_set_deinterlace ) ;
      DECL_P_FNC( libvlc_video_set_key_input ) ;
      DECL_P_FNC( libvlc_set_exit_handler ) ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      libvlc_instance_t *get_inst() { return( m_inst ) ; }
      const char *get_event_type_name( libvlc_event_type_t event_type ) const
      {  return( m_libvlc_event_type_name( event_type ) ) ; }
      const char *get_version() const ;
      void set_user_agent( const char *p_c_name, const char *p_c_http )
      {  m_libvlc_set_user_agent( m_inst, p_c_name, p_c_http ) ; }
      void set_app_id( const char *p_c_id, const char *p_c_version,
                       const char *p_c_icon
                     )
      {  m_libvlc_set_app_id( m_inst, p_c_id, p_c_version, p_c_icon ) ; }
      bool init_version() ;
      static wxString st_version_to_string( int i_version )
      {  return( wxString::Format( "%d", i_version ) ) ; }
      wxString version_to_string() const
      {  return( st_version_to_string( m_i_version ) ) ; }
      static wxString st_version_min_to_string()
      {  return( st_version_to_string( m_st_co_i_version_min ) ) ; }
      bool is_version_ok( const int &i_version ) const
      {  return( m_i_version >= i_version ) ; }
      bool is_version_min_ok() const
      {  return( is_version_ok( m_st_co_i_version_min ) ) ; }
      bool is_ok() const
      {  return( IsLoaded() && is_version_min_ok() ) ; }
      wxString build_init_string( const CChannel *p_channel )
      {  return( st_build_init_string( p_channel, m_i_version ) ) ; }
      static wxString st_build_init_string( const CChannel *p_channel,
                                            int i_vlc_version
                                          ) ;
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                       End of file CVlcInterface.h                         !
+==========================================================================*/
