/*====================================+=====================================+
! File CDlgAbout.cpp                  ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/hyperlink.h>
#include <wx/wrapsizer.h>
#include "CApp.h"
#include "CFrame.h"
#include "CDlgAbout.h"
#include "matelefree.h"
#include "matelefree_info.h"
#include "matelefree.xpm"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CDlgAbout::CDlgAbout( wxWindow *parent )
         : wxDialog( parent, wxID_ANY,
                     wxGetStockLabel( wxID_ABOUT, wxSTOCK_NOFLAGS )
                   )
{
   /*----------------------------------------------------------------------*/
   const wxString co_s_version(   MTF_VERSION " ("
                                + wxGetTranslation( MTF_BITNESS_TEXT )
                                + ")"
                              ) ;
   /*--( Build UI )--------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;
   wxBoxSizer *bsz_icon_ident = new wxBoxSizer( wxHORIZONTAL ) ;

   /*--( Icon )------------------------------------------------------------*/
   wxStaticBitmap *stb_icon
   = new wxStaticBitmap( this, wxID_ANY, wxBitmap( matelefree_xpm ) ) ;
   bsz_icon_ident->Add( stb_icon, 0, wxALL, 5 ) ;

   /*--( Identification )--------------------------------------------------*/
   wxBoxSizer *bsz_ident = new wxBoxSizer( wxVERTICAL ) ;
   wxStaticText *stx_name
   = new wxStaticText( this, wxID_ANY, MTF_DISPLAY_NAME ) ;
   stx_name->SetFont( wxFont( 25, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL,
                              wxFONTWEIGHT_BOLD
                            )
                    ) ;
   bsz_ident->Add( stx_name, 0, wxLEFT | wxRIGHT, 5 ) ;

   wxStaticText *stx_version
   = new wxStaticText( this, wxID_ANY, co_s_version ) ;
   stx_version->SetToolTip( wxString::Format( _( "Object type : %s" ),
                                              MTF_BUILT_TYPE
                                            )
                          ) ;
   stx_version->SetFont( wxFont( 8, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL,
                                 wxFONTWEIGHT_NORMAL
                               )
                       ) ;
   bsz_ident->Add( stx_version, 0, wxLEFT | wxRIGHT, 5 ) ;

   wxStaticText *stx_copyright
   = new wxStaticText( this, wxID_ANY, MTF_COPYRIGHT ) ;
   stx_copyright->SetFont( stx_version->GetFont() ) ;
   bsz_ident->Add( stx_copyright, 0, wxLEFT | wxRIGHT, 5 ) ;
   /*----------------------------------------------------------------------*/
   bsz_icon_ident->Add( bsz_ident, 0, wxEXPAND, 5 ) ;
   bsz_gen->Add( bsz_icon_ident, 0, wxALL | wxEXPAND, 10 ) ;

   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_detail = new wxBoxSizer( wxVERTICAL ) ;
   /*--( General Information )---------------------------------------------*/
   wxBoxSizer *bsz_info = new wxBoxSizer( wxVERTICAL ) ;

   bsz_info->Add( new wxStaticText( this, wxID_ANY,
                                    wxGetTranslation( MTF_DESCRIPTION )
                                  ),
                  0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT, 5
                ) ;
   bsz_detail->Add( bsz_info, 0, wxEXPAND | wxBOTTOM | wxLEFT | wxRIGHT, 5 );

   /*--( Licence )---------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_lic
   = new wxStaticBoxSizer( wxVERTICAL, this, _( "Licence" ) ) ;
   stbsz_lic->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                        wxGetTranslation( MTF_LICENSE ),
                                        MTF_LICENSE_URL
                                      ),
                   0, wxBOTTOM | wxLEFT | wxRIGHT, 5
                 ) ;
   bsz_detail->Add( stbsz_lic, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Web )-------------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_web
                  = new wxStaticBoxSizer( wxVERTICAL, this, _( "Web" ) ) ;
   wxFlexGridSizer *fgsz_web = new wxFlexGridSizer( 2, 0, 0 ) ;
   fgsz_web->AddGrowableCol( 1 ) ;

   fgsz_web->Add( new wxStaticText( this, wxID_ANY, _( "Site :" ) ),
                  0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5
                ) ;
   fgsz_web->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                       MTF_URL_SITE, MTF_URL_SITE
                                     ),
                  0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5
                ) ;
   fgsz_web->Add( new wxStaticText( this, wxID_ANY, _( "Forum :" ) ),
                  0, wxALIGN_CENTER_VERTICAL | wxBOTTOM | wxLEFT | wxRIGHT, 5
                ) ;
   fgsz_web->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                       MTF_URL_FORUM, MTF_URL_FORUM
                                     ),
                  0, wxALIGN_CENTER_VERTICAL | wxBOTTOM | wxRIGHT, 5
                ) ;

   stbsz_web->Add( fgsz_web, 0, wxEXPAND ) ;

   bsz_detail->Add( stbsz_web, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Remarks )---------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_rem
   = new wxStaticBoxSizer( wxVERTICAL, this, _( "Remarks and suggestions" ));

   stbsz_rem->Add( new wxHyperlinkCtrl( this, wxID_ANY, MTF_MAIL,
                                       "mailto:" + MTF_MAIL
                                     + "?subject=" MTF_DISPLAY_NAME_ASCII " "
                                     + co_s_version
                                     + " - " + wxGetOsDescription()
                                      ),
                   0, wxBOTTOM | wxLEFT | wxRIGHT, 5
                 ) ;
   bsz_detail->Add( stbsz_rem, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Translation )-----------------------------------------------------*/
   wxStaticBoxSizer *stbsz_trans
   = new wxStaticBoxSizer( wxVERTICAL, this, _( "Translation" ) ) ;

   /*--( The translator name is taken from translation file )--------------*/
   wxString s_mail_translator
          = wxTranslations::Get()->GetHeaderValue( "Language-Team" ) ;
   if( s_mail_translator.empty() )
   {  s_mail_translator = MTF_MAIL_TRANSLATOR ; }
   stbsz_trans->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                          s_mail_translator,
                                            "mailto:" + s_mail_translator
                                          + "?subject=Translation "
                                          + MTF_DISPLAY_NAME_ASCII " "
                                          + co_s_version
                                          + " - " + wxGetOsDescription()
                                        ),
                     0, wxBOTTOM | wxLEFT | wxRIGHT, 5
                   ) ;

   bsz_detail->Add( stbsz_trans, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Credits )---------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_cred
   = new wxStaticBoxSizer( wxVERTICAL, this, _( "Credits" ) ) ;
   wxWrapSizer *wsz = new wxWrapSizer() ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "Videolan", "www.videolan.org"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "wxWidgets", "www.wxwidgets.org"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "Tango Desktop Project",
                                  "tango.freedesktop.org"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "libcurl",
                                  "curl.haxx.se/libcurl"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "Code::Blocks", "codeblocks.org"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "Poedit", "poedit.net"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY,
                                  "Kodi Live", "kodilive.eu"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxHyperlinkCtrl( this, wxID_ANY, "Catch-up TV and More",
                                  "github.com/Catch-up-TV-and-More"
                                )
           ) ;
   wsz->AddSpacer( 10 ) ;
   wsz->Add( new wxStaticText( this, wxID_ANY, "Gilles" ), 0,
             wxALIGN_CENTER_VERTICAL | wxLEFT, 5
           ) ;

   stbsz_cred->Add( wsz, 0, wxBOTTOM | wxLEFT | wxRIGHT, 5 ) ;
   bsz_detail->Add( stbsz_cred, 1, wxALL | wxEXPAND, 5 ) ;
   /*----------------------------------------------------------------------*/
   bsz_gen->Add( bsz_detail, 0, wxEXPAND, 5 ) ;
   /*----------------------------------------------------------------------*/
   wxButton *bt_ok = new wxButton( this, wxID_OK ) ;
   bsz_gen->Add( bt_ok, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5 ) ;
   bt_ok->SetFocus() ;
   bt_ok->SetDefault() ;
   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
   Layout() ;
   SetInitialSize() ;
   CentreOnParent() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgAbout::get_modal()
{
   /*----------------------------------------------------------------------*/
   return( ShowModal() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file CDlgAbout.cpp                          !
+==========================================================================*/
