/*====================================+=====================================+
! File CDlgPrefs.cpp                  ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/spinctrl.h>
#include <wx/slider.h>
#include <wx/statline.h>
#include <wx/dir.h>
#include "CFrame.h"
#include "CDlgPrefs.h"
#include "CVlcInterface.h"
#include "matelefree_info.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CPanPrefsGen::CPanPrefsGen( wxWindow *parent, CDlgPrefs *dlg )
            : CPanPrefs( parent, dlg )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*--( Path )------------------------------------------------------------*/
   m_txt_vlc_path->SetValue( wxGetApp().M_s_vlc_path.get() ) ;
   m_txt_vlc_path->Bind( wxEVT_TEXT, &CPanPrefsGen::OnVlcPathChanged, this );
   m_bt_vlc_path_dlg->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                            &CPanPrefsGen::OnChooseVlcPath, this
                          ) ;
   m_txt_snapshot_path->SetValue( wxGetApp().M_s_snapshot_path.get() ) ;
   m_txt_snapshot_path->Bind( wxEVT_TEXT,
                              &CPanPrefsGen::OnSnapshotPathChanged, this
                            ) ;
   m_bt_snapshot_path_dlg->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                                 &CPanPrefsGen::OnChooseSnapshotPath, this
                               ) ;
   /*--( Lang )------------------------------------------------------------*/
   init_cho_lang() ;
   /*--( Freebox )---------------------------------------------------------*/
   m_chk_freebox->SetValue( wxGetApp().M_boo_freebox.get() ) ;
   m_chk_startup_play->SetValue( wxGetApp().M_boo_startup_play.get() );

   /*--( Update )----------------------------------------------------------*/
   m_cho_chkupd_freq->Append( _( "never" ) ) ;   // CUF_NEVER
   m_cho_chkupd_freq->Append( _( "daily" ) ) ;   // CUF_DAY
   m_cho_chkupd_freq->Append( _( "weekly" ) ) ;  // CUF_WEEK
   m_cho_chkupd_freq->Append( _( "monthly" ) ) ; // CUF_MONTH
   m_cho_chkupd_freq->SetSelection( wxGetApp().M_i_chkupd_freq.get() ) ;
   m_cho_chkupd_freq->Bind( wxEVT_CHOICE,
                            [ this ]( wxCommandEvent &event )
                            {  init_chkupd_next() ; }
                          ) ;
   init_chkupd_next() ;
   /*--( Stream quality )--------------------------------------------------*/
   m_cho_def_quality->Append( _( "Default" ) ) ;
   m_cho_def_quality->Append( str_stream::m_st_as_quality ) ;
     int i_quality
   = m_cho_def_quality->FindString( wxGetApp().M_s_def_quality.get() ) ;
   if( i_quality == wxNOT_FOUND )
   {  i_quality = 0 ; }
   m_cho_def_quality->SetSelection( i_quality ) ;
   /*----------------------------------------------------------------------*/
   m_i_max_nb_media->SetValue( wxGetApp().M_i_max_nb_media.get() ) ;
   m_chk_new_media_active->SetValue( wxGetApp().M_boo_new_media_active.get()
                                   ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::OnVlcPathChanged( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   check_vlc_path() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::OnSnapshotPathChanged( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   wxString s_path = m_txt_snapshot_path->GetValue() ;
   /*----------------------------------------------------------------------*/
   if( !s_path.empty() && !wxDir::Exists( s_path ) )
   {  /*-------------------------------------------------------------------*/
      m_txt_snapshot_path->SetForegroundColour( *wxRED ) ;
      m_txt_snapshot_path->Refresh() ;
      return ;
      /*-------------------------------------------------------------------*/
   }
   /*--( Path ok )---------------------------------------------------------*/
   m_txt_snapshot_path->SetForegroundColour( wxNullColour ) ;
   m_txt_snapshot_path->Refresh() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::OnChooseVlcPath( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   wxDirDialog dlg( this, _( "VLC path" ), wxEmptyString,
                    wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST
                  ) ;
   /*----------------------------------------------------------------------*/
   if( dlg.ShowModal() == wxID_OK )
   {  m_txt_vlc_path->SetValue( dlg.GetPath() ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::OnChooseSnapshotPath( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   wxDirDialog dlg( this, _( "Snapshot path" ), wxEmptyString,
                    wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST
                  ) ;
   /*----------------------------------------------------------------------*/
   if( dlg.ShowModal() == wxID_OK )
   {  m_txt_snapshot_path->SetValue( dlg.GetPath() ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::build_ui()
{
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;

   /*--( VLC Path )--------------------------------------------------------*/
   wxFlexGridSizer *fgsz_vlc_path = new wxFlexGridSizer( 3, 0, 0 ) ;
   fgsz_vlc_path->AddGrowableCol( 1 ) ;
   fgsz_vlc_path->Add( new wxStaticText( this, wxID_ANY, _( "VLC path" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_path = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                    wxDefaultPosition, wxSize( 400, -1 ), 0
                                  ) ;
   m_txt_vlc_path->SetToolTip( _( "Directory where VLC is installed.\n"
                                  "An empty value means using system "
                                  "defaults."
                                )
                             ) ;
   m_txt_vlc_path->AutoCompleteDirectories() ;
   fgsz_vlc_path->Add( m_txt_vlc_path, 1, wxEXPAND | wxALL, 5 ) ;
     m_bt_vlc_path_dlg
   = new wxButton( this, wxID_ANY, _( "..." ),
                   wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT
                 ) ;
   fgsz_vlc_path->Add( m_bt_vlc_path_dlg, 0, wxALL, 5 ) ;

   fgsz_vlc_path->AddStretchSpacer( 1 ) ;
   m_stx_vlc_info = new wxStaticText( this, wxID_ANY, wxEmptyString,
                                      wxDefaultPosition, wxDefaultSize,
                                      wxST_ELLIPSIZE_END
                                    ) ;
   fgsz_vlc_path->Add( m_stx_vlc_info, 1, wxEXPAND | wxALL, 5 ) ;
   fgsz_vlc_path->AddStretchSpacer( 1 ) ;

   bsz_gen->Add( fgsz_vlc_path ) ;

   /*--( Snapshot Path )---------------------------------------------------*/
   wxBoxSizer *bsz_snp_path = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_snp_path->Add( new wxStaticText( this, wxID_ANY,_( "Snapshot path" )),
                      0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                    ) ;
   m_txt_snapshot_path = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                         wxDefaultPosition, wxDefaultSize, 0
                                       ) ;
   m_txt_snapshot_path->AutoCompleteDirectories() ;
   m_txt_snapshot_path->SetToolTip(
                    _( "Directory where the channel snapshots are saved.\n"
                       "An empty value means using the executable directory."
                     )
                                  ) ;
   bsz_snp_path->Add( m_txt_snapshot_path, 1, wxEXPAND | wxALL, 5 ) ;

     m_bt_snapshot_path_dlg
   = new wxButton( this, wxID_ANY, _( "..." ),
                   wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT
                 ) ;
   bsz_snp_path->Add( m_bt_snapshot_path_dlg, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_snp_path, 0, wxEXPAND ) ;

   /*----------------------------------------------------------------------*/
   bsz_gen->Add( new wxStaticLine( this ), 0, wxEXPAND | wxALL, 5 ) ;

   /*--( Language )--------------------------------------------------------*/
   wxBoxSizer *bsz_lang = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_lang->Add( new wxStaticText( this, wxID_ANY, _( "Language" ) ),
                  0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                ) ;
   m_cho_lang = new wxChoice( this, wxID_ANY ) ;

   bsz_lang->Add( m_cho_lang, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_lang ) ;

   /*--( Freebox )---------------------------------------------------------*/
   m_chk_freebox = new wxCheckBox( this, wxID_ANY,
                                   _("Enable Freebox access" )
                                 ) ;
   m_chk_freebox->SetToolTip( _( "Check this option if you have access to a "
                                 "Freebox"
                               )
                            ) ;
   bsz_gen->Add( m_chk_freebox, 0, wxALL, 5 ) ;

   /*--( Startup Play )----------------------------------------------------*/
     m_chk_startup_play
   = new wxCheckBox( this, wxID_ANY,
                     _( "At startup, restore played channels" )
                   ) ;
   m_chk_startup_play->SetToolTip( _( "During startup, automatically play "
                                      "channels that were played during "
                                      "last program shutdown"
                                    )
                                 ) ;
   bsz_gen->Add( m_chk_startup_play, 0, wxALL, 5 ) ;

   /*--( Update )----------------------------------------------------------*/
   wxBoxSizer *bsz_upd = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_upd->Add( new wxStaticText( this, wxID_ANY, _( "Check for update" ) ),
                 0, wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
   m_cho_chkupd_freq = new wxChoice( this, wxID_ANY ) ;
   bsz_upd->Add( m_cho_chkupd_freq, 0, wxALL, 5 ) ;
   m_stx_chkupd_next = new wxStaticText( this, wxID_ANY, wxEmptyString ) ;
   bsz_upd->Add( m_stx_chkupd_next, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 ) ;
   bsz_gen->Add( bsz_upd ) ;

   /*--( Stream Quality )--------------------------------------------------*/
   wxBoxSizer *bsz_qual = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_qual->Add( new wxStaticText( this, wxID_ANY,
                                    _( "Default stream quality" )
                                  ),
                  0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                ) ;
   m_cho_def_quality = new wxChoice( this, wxID_ANY ) ;
   bsz_qual->Add( m_cho_def_quality, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_qual ) ;

   /*----------------------------------------------------------------------*/
   bsz_gen->Add( new wxStaticLine( this ), 0, wxEXPAND | wxALL, 5 ) ;

   /*--( Maximum number of players before alerting the user )--------------*/
   wxBoxSizer *bsz_mnp = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_mnp->Add( new wxStaticText( this, wxID_ANY,
                 _( "Number of simultaneous play before alert (0=no limit)" )
                                 ),
                 0, wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
     m_i_max_nb_media
   = new wxSpinCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition,
                     wxSize( 80, -1 ), wxSP_ARROW_KEYS, 0, 999999, 0
                   ) ;
   bsz_mnp->Add( m_i_max_nb_media, 0,
                 wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
   bsz_gen->Add( bsz_mnp, 0, wxEXPAND ) ;

   /*--( New player becomes the active one ? )-----------------------------*/
     m_chk_new_media_active
   = new wxCheckBox( this, wxID_ANY,
                     _( "Select the new player after it has been added" )
                   ) ;
   m_chk_new_media_active->SetToolTip( _( "When a new player is added it "
                                          "is automatically selected"
                                        )
                                     ) ;
   bsz_gen->Add( m_chk_new_media_active, 0, wxALL, 5 ) ;

   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_default()
{
   /*--( VLC )-------------------------------------------------------------*/
   m_txt_vlc_path->SetValue( wxGetApp().M_s_vlc_path.get_def() ) ;
   m_txt_snapshot_path->SetValue( wxGetApp().M_s_snapshot_path.get_def() ) ;
   /*--( Lang : default is the first one )---------------------------------*/
   m_cho_lang->SetSelection( 0 ) ;
   /*--( Freebox )---------------------------------------------------------*/
   m_chk_freebox->SetValue( wxGetApp().M_boo_freebox.get_def() ) ;
   m_chk_startup_play->SetValue( wxGetApp().M_boo_startup_play.get_def() ) ;
   /*--( Update : default is the first one )-------------------------------*/
   m_cho_chkupd_freq->SetSelection( wxGetApp().M_i_chkupd_freq.get_def() ) ;
   /*--( Stream quality )--------------------------------------------------*/
     int i_quality
   = m_cho_def_quality->FindString( wxGetApp().M_s_def_quality.get_def() ) ;
   if( i_quality == wxNOT_FOUND )
   {  i_quality = 0 ; }
   m_cho_def_quality->SetSelection( i_quality ) ;

   /*----------------------------------------------------------------------*/
   m_i_max_nb_media->SetValue( wxGetApp().M_i_max_nb_media.get_def() ) ;
   m_chk_new_media_active->SetValue(
                                  wxGetApp().M_boo_new_media_active.get_def()
                                   ) ;
   /*--( The default path has to be checked )------------------------------*/
   check_vlc_path() ;
   /*----------------------------------------------------------------------*/
}

/*--( Returns "true" if a restart has to be done )-------------------------*/
bool CPanPrefsGen::apply_changes()
{
   /*----------------------------------------------------------------------*/
   bool boo_restart = false ;
   /*----------------------------------------------------------------------*/
   wxString s_vlc_path = m_txt_vlc_path->GetValue() ;
   if( wxGetApp().M_s_vlc_path.get() != s_vlc_path )
   {  /*-------------------------------------------------------------------*/
      wxGetApp().M_s_vlc_path.set( s_vlc_path ) ;
      /*--( Use this new path to start VLC )-------------------------------*/
      wxGetApp().init_fn_vlc() ;
      /*--( A restart has to be done if some vlc int are allocated ! )-----*/
      boo_restart = !wxGetApp().m_frame->get_vec_media().empty() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_s_snapshot_path.set( m_txt_snapshot_path->GetValue() ) ;
   /*----------------------------------------------------------------------*/
   wxString s_lang = get_cho_lang_sel_canon() ;
   if( wxGetApp().M_s_lang.get() != s_lang )
   {  wxGetApp().M_s_lang.set( s_lang ) ; boo_restart = true ; }
   /*----------------------------------------------------------------------*/
   bool boo_freebox = m_chk_freebox->GetValue() ;
   if( wxGetApp().M_boo_freebox.get() != boo_freebox )
   {  wxGetApp().M_boo_freebox.set( boo_freebox ) ; boo_restart = true ; }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_boo_startup_play.set( m_chk_startup_play->GetValue() ) ;
   /*----------------------------------------------------------------------*/
   wxGetApp().M_i_chkupd_freq.set( m_cho_chkupd_freq->GetSelection() ) ;
   /*----------------------------------------------------------------------*/
   if( m_cho_def_quality->GetSelection() == 0 )
   {  wxGetApp().M_s_def_quality.clear() ; }
   else
   {  wxGetApp().M_s_def_quality.set( m_cho_def_quality->GetStringSelection()
                                    ) ;
   }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_i_max_nb_media.set( m_i_max_nb_media->GetValue() ) ;
   wxGetApp().M_boo_new_media_active.set( m_chk_new_media_active->GetValue()
                                        ) ;
   /*----------------------------------------------------------------------*/
   return( boo_restart ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::check_vlc_path()
{
   /*----------------------------------------------------------------------*/
   wxString s_path = m_txt_vlc_path->GetValue() ;
   /*----------------------------------------------------------------------*/
   if( !s_path.empty() && !wxDir::Exists( s_path ) )
   {
      /*-------------------------------------------------------------------*/
      m_txt_vlc_path->SetForegroundColour( *wxRED ) ;
      m_txt_vlc_path->Refresh() ;
      m_stx_vlc_info->SetLabelText( wxEmptyString ) ;
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   m_txt_vlc_path->SetForegroundColour( wxNullColour ) ;
   m_txt_vlc_path->Refresh() ;
   /*----------------------------------------------------------------------*/
   wxString s_info ;
   CVlcInterface *p_vlc_int ;
   /*----------------------------------------------------------------------*/
   {  wxBusyCursor tell_user_we_are_working ;
      m_stx_vlc_info->SetForegroundColour( *wxBLUE ) ;
      m_stx_vlc_info->SetLabelText( _( "Checking ..." ) ) ;
      p_vlc_int = new CVlcInterface( s_path, nullptr ) ;
   }
   /*----------------------------------------------------------------------*/
   if( p_vlc_int->IsLoaded() )
   {  /*-------------------------------------------------------------------*/
      s_info = _( "libvlc found" ) + " : " + p_vlc_int->get_version() ;
      /*-------------------------------------------------------------------*/
      if( !p_vlc_int->is_version_min_ok() )
      {  /*----------------------------------------------------------------*/
         s_info += ". " + wxString::Format( _( "At least %s is required." ),
                                    CVlcInterface::st_version_min_to_string()
                                          ) ;
         /*----------------------------------------------------------------*/
         m_stx_vlc_info->SetForegroundColour( *wxRED ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      m_stx_vlc_info->SetLabelText( s_info ) ;
      /*-------------------------------------------------------------------*/
   }
   else
   {  /*-------------------------------------------------------------------*/
      #ifdef __WXMSW__
         /*--( Check bitness if path is specified )------------------------*/
         if( !s_path.empty() )
         {  /*-------------------------------------------------------------*/
            int i_libvlc_bitness = mtf::msw_exe_bitness(
                             wxFileName( s_path, "libvlc.dll" ).GetFullPath()
                                                    ) ;
            /*-------------------------------------------------------------*/
            if( i_libvlc_bitness > 0 && i_libvlc_bitness != MTF_BITNESS_NUM )
            {  /*----------------------------------------------------------*/
               s_info.Printf( _( "The \"libvlc\" found is %d-bit. "
                                 "A %d-bit program can't load it"
                               ),
                              i_libvlc_bitness, MTF_BITNESS_NUM
                            ) ;
               /*----------------------------------------------------------*/
            }
            /*-------------------------------------------------------------*/
         }
      #endif
      /*--( Default message )----------------------------------------------*/
      if( s_info.empty() ) { s_info = _( "\"libvlc\" can't be loaded" ) ; }
      /*-------------------------------------------------------------------*/
      m_stx_vlc_info->SetForegroundColour( *wxRED ) ;
      m_stx_vlc_info->SetLabelText( s_info ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   delete p_vlc_int ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::append_cho_lang( const wxString &s_desc,
                                    const wxString &s_canon
                                  )
{  m_cho_lang->Append( s_desc, new wxStringClientData( s_canon ) ) ; }

/*-------------------------------------------------------------------------*/
wxString CPanPrefsGen::get_cho_lang_sel_canon()
{
   /*----------------------------------------------------------------------*/
   int  i_sel = m_cho_lang->GetCurrentSelection() ;
   void *p_object = m_cho_lang->GetClientObject( i_sel ) ;
   /*----------------------------------------------------------------------*/
   return( p_object == nullptr
           ? wxString()
           : reinterpret_cast< wxStringClientData * >( p_object )->GetData()
         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_cho_lang()
{
   /*----------------------------------------------------------------------*/
   wxArrayString     as_canon       ;
   mtf::t_map_strstr map_desc_canon ;
   size_t            sz_lang_sel    ;

   /*----------------------------------------------------------------------*/
     as_canon
   = wxTranslations::Get()->GetAvailableTranslations( MTF_NAME ) ;
   /*--( Load the couples into a map to get it sorted on description )-----*/
   for( const auto &s_canon : as_canon )
   {
      /*-------------------------------------------------------------------*/
      const wxLanguageInfo *p_li = wxLocale::FindLanguageInfo( s_canon ) ;
      if( p_li != nullptr )
      {  map_desc_canon[ wxGetTranslation( p_li->Description ) ] = s_canon ;}
      /*-------------------------------------------------------------------*/
   }

   /*--( The default will be displayed first )-----------------------------*/
   sz_lang_sel = m_cho_lang->GetCount() ;
   m_cho_lang->Append( _( "System (if translation exists)" ) ) ;

   /*--( Then the couples sorted on description (sort done by "map") )-----*/
   for( const auto &desc_canon : map_desc_canon )
   {
      /*--( Language chosen by the user ? (non default one) )--------------*/
      if(    !wxGetApp().M_s_lang.empty()
          && wxGetApp().M_s_lang.get() == desc_canon.second
        )
      {  sz_lang_sel = m_cho_lang->GetCount() ; }
      /*-------------------------------------------------------------------*/
      append_cho_lang( desc_canon.first, desc_canon.second ) ;
      /*-------------------------------------------------------------------*/
   }

   /*-----------------------------------------------------------------------+
   ! Finally the "no translation". It is mainly defined for test purpose.   !
   ! The idea is to use a code that is not defined and so use the           !
   ! "internal" text.                                                       !
   +-----------------------------------------------------------------------*/
   const wxString s_lang_none( _( "No translation (English)" ) ) ;
   const wxString s_lang_none_code( "**none**" ) ;
   /*----------------------------------------------------------------------*/
   if(    !wxGetApp().M_s_lang.empty()
       && wxGetApp().M_s_lang.get() == s_lang_none_code
     )
   {  sz_lang_sel = m_cho_lang->GetCount() ; }
   /*----------------------------------------------------------------------*/
   append_cho_lang( s_lang_none, s_lang_none_code ) ;
   /*----------------------------------------------------------------------*/
   m_cho_lang->SetSelection( sz_lang_sel ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_chkupd_next()
{
   /*----------------------------------------------------------------------*/
   wxString s_next ;
   /*----------------------------------------------------------------------*/
   if( m_cho_chkupd_freq->GetSelection() == CUF_NEVER )
   {  s_next = _( "never" ) ; }
   else
   {  /*-------------------------------------------------------------------*/
      wxString s_date_next
             = wxGetApp().next_chkupd( m_cho_chkupd_freq->GetSelection() ) ;
      wxDateTime dt_next ;
      /*-------------------------------------------------------------------*/
      if( s_date_next.empty() || !dt_next.ParseISODate( s_date_next ) )
      {  s_next = _( "unknown" ) ; }
      else
      {  s_next = dt_next.Format( "%A %d %B" ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   m_stx_chkupd_next->SetLabelText( wxString::Format( _( "next check : %s" ),
                                                      s_next
                                                    )
                                  ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPanPrefsAdv::CPanPrefsAdv( wxWindow *parent, CDlgPrefs *dlg )
            : CPanPrefs( parent, dlg )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*--( VLC init string parameters )--------------------------------------*/
   m_txt_vlc_init_dm->SetValue( wxGetApp().M_s_vlc_init_dm.get() ) ;
   m_txt_vlc_init_tw->SetValue( wxGetApp().M_s_vlc_init_tw.get() ) ;
   m_txt_vlc_init_yt->SetValue( wxGetApp().M_s_vlc_init_yt.get() ) ;
   m_txt_vlc_init_us->SetValue( wxGetApp().M_s_vlc_init_us.get() ) ;
   m_txt_vlc_init_pe->SetValue( wxGetApp().M_s_vlc_init_pe.get() ) ;
   m_txt_vlc_init_mx->SetValue( wxGetApp().M_s_vlc_init_mx.get() ) ;
   m_txt_vlc_init_other->SetValue( wxGetApp().M_s_vlc_init_other.get() ) ;
   m_txt_vlc_init_all->SetValue( wxGetApp().M_s_vlc_init_all.get() ) ;
   /*--( Deinterlace modes )-----------------------------------------------*/
   m_cho_deinterlace->Append( _( "None" ) ) ;
   m_cho_deinterlace->Append( "blend" ) ;
   m_cho_deinterlace->Append( "bob" ) ;
   m_cho_deinterlace->Append( "discard" ) ;
   m_cho_deinterlace->Append( "linear" ) ;
   m_cho_deinterlace->Append( "mean" ) ;
   m_cho_deinterlace->Append( "x" ) ;
   m_cho_deinterlace->Append( "yadif" ) ;
   m_cho_deinterlace->Append( "yadif2x" ) ;
   m_cho_deinterlace->Append( "phosphor" ) ;
   m_cho_deinterlace->Append( "ivtc" ) ;
     int i_mode
   = m_cho_deinterlace->FindString( wxGetApp().M_s_deinterlace.get() ) ;
   if( i_mode == wxNOT_FOUND )
   {  i_mode = 0 ; }
   m_cho_deinterlace->SetSelection( i_mode ) ;
   /*--( Advanced mute )---------------------------------------------------*/
   m_chk_mute_deact->SetValue( wxGetApp().M_boo_mute_deact.get() ) ;
   m_i_check_media_delay->SetValue( wxGetApp().M_i_check_media_delay.get() );
   m_i_viewers_delay->SetValue( wxGetApp().M_i_viewers_delay.get() ) ;
   m_i_retry_delay->SetValue( wxGetApp().M_i_retry_delay.get() ) ;
   /*----------------------------------------------------------------------*/
   m_txt_twitch_client_id->SetValue( wxGetApp().M_s_twitch_client_id.get() );
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsAdv::build_ui()
{
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;

   /*--( VLC initialization string )---------------------------------------*/
   wxStaticBoxSizer *stbsz_vlc_init
       = new wxStaticBoxSizer( wxVERTICAL, this,
                               _( "VLC initialization parameters by source" )
                             ) ;
   wxFlexGridSizer *fgsz_vlc_init = new wxFlexGridSizer( 2, 0, 0 ) ;
   fgsz_vlc_init->AddGrowableCol( 1 ) ;
   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "Dailymotion" )),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_dm = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                       wxDefaultPosition, wxDefaultSize, 0
                                     ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_dm, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "Twitch" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_tw = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                       wxDefaultPosition, wxDefaultSize, 0
                                     ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_tw, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "Youtube" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_yt = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                       wxDefaultPosition, wxDefaultSize, 0
                                     ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_yt, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "Ustream" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_us = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                       wxDefaultPosition, wxDefaultSize, 0
                                     ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_us, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "Periscope" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_pe = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                       wxDefaultPosition, wxDefaultSize, 0
                                     ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_pe, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "Mixer" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_mx = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                       wxDefaultPosition, wxDefaultSize, 0
                                     ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_mx, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY, _( "The others" ) ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_other = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                          wxDefaultPosition, wxDefaultSize, 0
                                        ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_other, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_vlc_init->Add( new wxStaticText( this, wxID_ANY,
                                         _( "All channels" )
                                       ),
                       0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                     ) ;
   m_txt_vlc_init_all = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                        wxDefaultPosition, wxDefaultSize, 0
                                      ) ;
   fgsz_vlc_init->Add( m_txt_vlc_init_all, 1, wxEXPAND | wxALL, 5 ) ;

   stbsz_vlc_init->Add( fgsz_vlc_init, 0, wxALL | wxEXPAND, 5 ) ;

   bsz_gen->Add( stbsz_vlc_init, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Deinterlace )-----------------------------------------------------*/
   wxBoxSizer *bsz_deint = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_deint->Add( new wxStaticText( this, wxID_ANY,
                                     _( "Deinterlace mode" )
                                   ),
                   0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                 ) ;
   m_cho_deinterlace = new wxChoice( this, wxID_ANY ) ;
   bsz_deint->Add( m_cho_deinterlace, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_deint ) ;

   /*--( Deactivate sound for mute )---------------------------------------*/
   m_chk_mute_deact = new wxCheckBox( this, wxID_ANY,
                                      _( "Deactivate audio when mute" )
                                    ) ;
   m_chk_mute_deact->SetToolTip(
                  _( "Muting audio will deactivate the audio output instead "
                     "of simply muting. This can be useful depending on the "
                     "way your system manages sound."
                   )
                                ) ;
   bsz_gen->Add( m_chk_mute_deact, 0, wxALL, 5 ) ;

   /*--( Check media delay )-----------------------------------------------*/
   wxBoxSizer *bsz_chmed = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_chmed->Add( new wxStaticText( this, wxID_ANY,
                      _( "Check media video and audio after (milliseconds)" )
                                   ),
                   0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                 ) ;
     m_i_check_media_delay
   = new wxSpinCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition,
                     wxSize( 80, -1 ), wxSP_ARROW_KEYS, 0, 999999, 0
                   ) ;
   bsz_chmed->Add( m_i_check_media_delay, 0,
                   wxALIGN_CENTER_VERTICAL | wxALL, 5
                 ) ;
   bsz_gen->Add( bsz_chmed, 0, wxEXPAND ) ;

   /*--( Time between two number of viewers refresh )----------------------*/
   wxBoxSizer *bsz_vide = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_vide->Add( new wxStaticText( this, wxID_ANY,
           _( "Time between refreshes of viewers number (seconds, 0=never)" )
                                  ),
                  0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                ) ;
     m_i_viewers_delay
   = new wxSpinCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition,
                     wxSize( 80, -1 ), wxSP_ARROW_KEYS, 0, 999999, 0
                   ) ;
   bsz_vide->Add( m_i_viewers_delay, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_vide ) ;

   /*--( Time between two get streams retries )----------------------------*/
   wxBoxSizer *bsz_rede = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_rede->Add( new wxStaticText( this, wxID_ANY,
                  _( "Time between retries to get channel streams (seconds)")
                                  ),
                  0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                ) ;
     m_i_retry_delay
   = new wxSpinCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition,
                     wxSize( 80, -1 ), wxSP_ARROW_KEYS, 0, 999999, 0
                   ) ;
   bsz_rede->Add( m_i_retry_delay, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_rede ) ;

   /*--( Twitch Client SID )-----------------------------------------------*/
   wxBoxSizer *bsz_twsid = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_twsid->Add( new wxStaticText( this, wxID_ANY,
                                     _( "Twitch Client SID" )
                                   ),
                   0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                 ) ;
     m_txt_twitch_client_id
   = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                     wxDefaultPosition, wxDefaultSize, 0
                   ) ;
   bsz_twsid->Add( m_txt_twitch_client_id, 1,
                   wxALIGN_CENTER_VERTICAL | wxALL, 5
                 ) ;
   bsz_gen->Add( bsz_twsid, 0, wxEXPAND ) ;

   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsAdv::init_default()
{
   /*--( VLC init string parameters )--------------------------------------*/
   m_txt_vlc_init_dm->SetValue( wxGetApp().M_s_vlc_init_dm.get_def() ) ;
   m_txt_vlc_init_tw->SetValue( wxGetApp().M_s_vlc_init_tw.get_def() ) ;
   m_txt_vlc_init_yt->SetValue( wxGetApp().M_s_vlc_init_yt.get_def() ) ;
   m_txt_vlc_init_us->SetValue( wxGetApp().M_s_vlc_init_us.get_def() ) ;
   m_txt_vlc_init_pe->SetValue( wxGetApp().M_s_vlc_init_pe.get_def() ) ;
   m_txt_vlc_init_mx->SetValue( wxGetApp().M_s_vlc_init_mx.get_def() ) ;
   m_txt_vlc_init_other->SetValue( wxGetApp().M_s_vlc_init_other.get_def() );
   m_txt_vlc_init_all->SetValue( wxGetApp().M_s_vlc_init_all.get_def() ) ;

   /*--( Deinterlace modes : default is the first one )--------------------*/
   m_cho_deinterlace->SetSelection( 0 ) ;

   /*--( Advanced mute )---------------------------------------------------*/
   m_chk_mute_deact->SetValue( wxGetApp().M_boo_mute_deact.get_def() ) ;
   m_i_check_media_delay->SetValue(
                                   wxGetApp().M_i_check_media_delay.get_def()
                                  ) ;
   m_i_viewers_delay->SetValue( wxGetApp().M_i_viewers_delay.get_def() ) ;
   m_i_retry_delay->SetValue( wxGetApp().M_i_retry_delay.get_def() ) ;
   /*----------------------------------------------------------------------*/
   m_txt_twitch_client_id->SetValue(
                                    wxGetApp().M_s_twitch_client_id.get_def()
                                   ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Returns "true" if a restart has to be done )-------------------------*/
bool CPanPrefsAdv::apply_changes()
{
   /*----------------------------------------------------------------------*/
   bool boo_restart = false ;
   /*----------------------------------------------------------------------*/
   wxString s_vlc_init = m_txt_vlc_init_dm->GetValue() ;
   if( wxGetApp().M_s_vlc_init_dm.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_dm.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_tw->GetValue() ;
   if( wxGetApp().M_s_vlc_init_tw.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_tw.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_yt->GetValue() ;
   if( wxGetApp().M_s_vlc_init_yt.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_yt.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_us->GetValue() ;
   if( wxGetApp().M_s_vlc_init_us.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_us.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_pe->GetValue() ;
   if( wxGetApp().M_s_vlc_init_pe.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_pe.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_mx->GetValue() ;
   if( wxGetApp().M_s_vlc_init_mx.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_mx.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_other->GetValue() ;
   if( wxGetApp().M_s_vlc_init_other.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_other.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   s_vlc_init = m_txt_vlc_init_all->GetValue() ;
   if( wxGetApp().M_s_vlc_init_all.get() != s_vlc_init )
   {  wxGetApp().M_s_vlc_init_all.set( s_vlc_init ) ; }
   /*----------------------------------------------------------------------*/
   if( m_cho_deinterlace->GetSelection() == 0 )
   {  wxGetApp().M_s_deinterlace.clear() ; }
   else
   {  wxGetApp().M_s_deinterlace.set( m_cho_deinterlace->GetStringSelection()
                                    ) ;
   }
   /*--( Setting the parameter to all currently running players )----------*/
   for( auto p_media : wxGetApp().m_frame->get_vec_media() )
   {  p_media->set_deinterlace() ; }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_boo_mute_deact.set( m_chk_mute_deact->GetValue() ) ;
   wxGetApp().M_i_check_media_delay.set( m_i_check_media_delay->GetValue() );
   /*----------------------------------------------------------------------*/
   int i_viewers_delay = m_i_viewers_delay->GetValue() ;
   if( wxGetApp().M_i_viewers_delay.get() != i_viewers_delay )
   {  /*-------------------------------------------------------------------*/
      wxGetApp().M_i_viewers_delay.set( i_viewers_delay ) ;
      wxGetApp().m_frame->restart_viewer_timer_if_started() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_i_retry_delay.set( m_i_retry_delay->GetValue() ) ;
   /*----------------------------------------------------------------------*/
   wxString s_twitch_client_id = m_txt_twitch_client_id->GetValue() ;
   if( wxGetApp().M_s_twitch_client_id.get() != s_twitch_client_id )
   {  /*-------------------------------------------------------------------*/
      wxGetApp().M_s_twitch_client_id.set( s_twitch_client_id ) ;
      boo_restart = true ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( boo_restart ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPanPrefsNet::CPanPrefsNet( wxWindow *parent, CDlgPrefs *dlg )
            : CPanPrefs( parent, dlg )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*----------------------------------------------------------------------*/
   m_txt_proxy_server->SetValue( wxGetApp().M_s_proxy_server.get() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsNet::build_ui()
{
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;

   /*----------------------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_proxy
              = new wxStaticBoxSizer( wxVERTICAL, this, _( "Proxy" ) ) ;
   wxFlexGridSizer *fgsz_proxy = new wxFlexGridSizer( 2, 0, 0 ) ;
   fgsz_proxy->AddGrowableCol( 1 ) ;
   fgsz_proxy->Add( new wxStaticText( this, wxID_ANY, _( "Server" ) ),
                    0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                  ) ;
   m_txt_proxy_server = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                        wxDefaultPosition, wxDefaultSize, 0
                                      ) ;


   fgsz_proxy->Add( m_txt_proxy_server, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_proxy->AddStretchSpacer( 1 ) ;
   fgsz_proxy->Add( new wxStaticText( this, wxID_ANY,
                         _( "The server address must follow the format :\n"
                            "[scheme://][username:password@]server[:port]\n"
                            "For example :\n"
                            "socks://127.0.0.1:8080\n"
                            "http://matelefree:monpwd@myproxy.net:10000"
                          )
                                    ),
                    0, wxLEFT | wxRIGHT | wxBOTTOM, 5
                  ) ;

   stbsz_proxy->Add( fgsz_proxy, 0, wxALL | wxEXPAND, 5 ) ;

   bsz_gen->Add( stbsz_proxy, 0, wxALL | wxEXPAND, 5 ) ;

   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsNet::init_default()
{
   /*----------------------------------------------------------------------*/
   m_txt_proxy_server->SetValue( wxGetApp().M_s_proxy_server.get_def() ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Returns "true" if a restart has to be done )-------------------------*/
bool CPanPrefsNet::apply_changes()
{
   /*----------------------------------------------------------------------*/
   bool boo_restart = false ;
   /*----------------------------------------------------------------------*/
   wxString s_proxy_server = m_txt_proxy_server->GetValue() ;
   if( wxGetApp().M_s_proxy_server.get() != s_proxy_server )
   {  wxGetApp().M_s_proxy_server.set( s_proxy_server ) ; }
   /*----------------------------------------------------------------------*/
   return( boo_restart ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgPrefs::CDlgPrefs()
         : wxPropertySheetDialog( this, wxID_ANY, wxEmptyString,
                                  wxDefaultPosition, wxDefaultSize,
                                  wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER
                                )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*----------------------------------------------------------------------*/
   mtf::init_tlw_rect( this, wxGetApp().M_prefs_rec ) ;
   Layout() ;
   /*--( To see vlc checking messages )------------------------------------*/
   Update() ;
   /*----------------------------------------------------------------------*/
   m_p_gen->check_vlc_path() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgPrefs::~CDlgPrefs()
{
   /*----------------------------------------------------------------------*/
   mtf::save_tlw_rect( this, wxGetApp().M_prefs_rec ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgPrefs::build_ui()
{
   /*--( Set window title using standard value )---------------------------*/
   SetTitle(   MTF_DISPLAY_NAME
             + " - " + wxGetStockLabel( wxID_PREFERENCES, wxSTOCK_NOFLAGS )
           ) ;
   /*----------------------------------------------------------------------*/
   wxBookCtrlBase *book = GetBookCtrl() ;
   /*----------------------------------------------------------------------*/
   m_p_gen = new CPanPrefsGen( book, this ) ;
   book->AddPage( m_p_gen, _( "General" ) ) ;
   book->AddPage( new CPanPrefsAdv( book, this ), _( "Advanced" ) ) ;
   book->AddPage( new CPanPrefsNet( book, this ), _( "Network" ) ) ;
   /*--( Help button is used to restore data to defaults )-----------------*/
   CreateButtons( wxOK | wxAPPLY | wxCANCEL | wxHELP ) ;
   /*----------------------------------------------------------------------*/
   wxButton *bt_ok = static_cast< wxButton * >( FindWindow( wxID_OK ) ) ;
   if( bt_ok != nullptr )
   {  bt_ok->Bind( wxEVT_BUTTON, &CDlgPrefs::OnBtOk, this ) ; }
   /*----------------------------------------------------------------------*/
   wxButton *bt_apply = static_cast< wxButton * >( FindWindow( wxID_APPLY ));
   if( bt_apply != nullptr )
   {  bt_apply->Bind( wxEVT_BUTTON, &CDlgPrefs::OnBtApply, this ) ; }
   /*----------------------------------------------------------------------*/
   wxButton *bt_reinit_page
          = static_cast< wxButton * >( FindWindow( wxID_HELP ) ) ;
   if( bt_reinit_page != nullptr )
   {  bt_reinit_page->SetLabel( _( "Reset current page" ) ) ;
      bt_reinit_page->Bind( wxEVT_BUTTON, &CDlgPrefs::OnBtResetPage, this ) ;
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgPrefs::apply_changes()
{
   /*--( Each panel will tell if a restart is needed ... )-----------------*/
   for( size_t sz_num = 0 ; sz_num < get_nb_page() ; ++sz_num )
   {
      /*-------------------------------------------------------------------*/
      if( get_page( sz_num )->apply_changes() )
      {  m_boo_restart = true ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgPrefs::get_modal()
{
   /*----------------------------------------------------------------------*/
   if( ShowModal() == wxID_OK )
   {  apply_changes() ; }
   /*----------------------------------------------------------------------*/
   if(    m_boo_restart
       && wxMessageBox( _( "Some changes need a restart to be active\n\n"
                           "Restart now ?"
                         ),
                         _( "Restart" ) + ' ' + MTF_DISPLAY_NAME + " ?",
                         wxYES_NO
                      ) == wxYES
     )
   {  wxGetApp().restart_on_exit() ; }

   /*----------------------------------------------------------------------*/
   return( GetReturnCode() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file CDlgPrefs.cpp                          !
+==========================================================================*/
