/*====================================+=====================================+
! File CWinFbtvoOk.cpp                ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include "wx/uri.h"
#include "wx/socket.h"
#include "CFrame.h"
#include "CWinFbtvoOk.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CWinFbtvoOk::CWinFbtvoOk( wxWindow *parent, CPlaylistFbtvoOk *pl_fbtvo_ok )
           : wxPanel( parent ),
             m_pl( pl_fbtvo_ok )
{
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz = new wxBoxSizer( wxVERTICAL ) ;
   /*----------------------------------------------------------------------*/
   m_bt_start = new wxButton( this, wxID_ANY, _( "Search" ) ) ;
   m_bt_start->SetToolTip( _( "Search for \"En option\" channels that "
                              "you can currently watch"
                            )
                         ) ;
   bsz->Add( m_bt_start, 0, wxCENTER | wxLEFT | wxTOP | wxRIGHT, 5 ) ;
   /*--( Add gauge )-------------------------------------------------------*/
   m_ga_progress = new wxGauge( this, wxID_ANY, 1, wxDefaultPosition,
                                wxSize( -1, 5 )
                              ) ;
   m_ga_progress->SetValue( 0 ) ;
   bsz->Add( m_ga_progress, 0, wxEXPAND | wxALL, 5 ) ;
   /*----------------------------------------------------------------------*/
     m_stx_msg
   = new wxStaticText( this, wxID_ANY, wxEmptyString,
                       wxDefaultPosition, wxDefaultSize,
                       wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE
                     ) ;
   msg_info( _( "\"En option\" accessible channels" ) ) ;
   bsz->Add( m_stx_msg, 0, wxEXPAND | wxLEFT | wxBOTTOM | wxRIGHT, 5 ) ;
   /*----------------------------------------------------------------------*/
   m_bt_start->Bind( wxEVT_BUTTON,
                     [ this ]( wxCommandEvent &event ) { start_search() ; }
                   ) ;
   /*----------------------------------------------------------------------*/
   SetSizer( bsz ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static bool st_create_freebox_rtsp_socket( const wxURI &uri,
                                           wxSocketClient &socket
                                         )
{
   /*----------------------------------------------------------------------*/
   long l_port ;
   /*----------------------------------------------------------------------*/
   if( socket.IsOk() && socket.IsConnected() )
   {  socket.Close() ; }
   /*----------------------------------------------------------------------*/
   if( uri.GetScheme() != "rtsp" )
   {  return( false ) ; }
   /*----------------------------------------------------------------------*/
   if( uri.GetPort().empty() || !uri.GetPort().ToLong( &l_port ) )
   {  l_port = 554 ; }
   /*----------------------------------------------------------------------*/
   wxIPV4address addr ;
   if( !addr.Hostname( uri.GetServer() ) || !addr.Service( l_port ) )
   {  return( false ) ; }
   /*----------------------------------------------------------------------*/
   socket.Connect( addr, false ) ;
   if( !socket.IsConnected() && !socket.WaitOnConnect( 5, 0 ) )
   {  return( false ) ; }
   /*----------------------------------------------------------------------*/
   return( true ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static bool st_check_rtsp_stream( wxSocketClient &socket,
                                  const wxString &s_url
                                )
{
   /*----------------------------------------------------------------------*/
   char tb_c_buffer[ 1024 ] ;
   int  i_length ;
   /*----------------------------------------------------------------------*/
   i_length = sprintf( tb_c_buffer,
                       "SETUP %s RTSP/1.0\r\n"
                       "CSeq: 1\r\n"
                       "Transport: RTP/AVP;unicast;client_port=9000-9001\r\n"
                       "\r\n",
                       s_url.mb_str().data()
                     ) ;
   /*--( Write command : no multiple write are expected )------------------*/
   socket.Write( tb_c_buffer, i_length ) ;
   if( socket.Error() || ( int )socket.LastWriteCount() != i_length )
   {  return( false ) ; }
   /*--( Read answer )-----------------------------------------------------*/
   socket.Read( tb_c_buffer, sizeof( tb_c_buffer ) ) ;
   if( socket.Error() || ( i_length = socket.LastReadCount() ) < 20 )
   {  return( false ) ; }

   /*--( If a session number is present the channel is accessible )--------*/
   char *p_c_session_start = strstr( tb_c_buffer, "\nSession: " ) ;
   if( p_c_session_start == nullptr ) { return( false ) ; }

   /*-----------------------------------------------------------------------+
   ! Now we have got to clean up this started session                       !
   +-----------------------------------------------------------------------*/

   /*--( Search for the end of the line after skipping the "\n" )----------*/
   char *p_c_session_end = strpbrk( ++p_c_session_start, "\r\n" ) ;
   if( p_c_session_end == nullptr ) { return( false ) ; }
   /*----------------------------------------------------------------------*/
   i_length = sprintf( tb_c_buffer,
                       "TEARDOWN %s RTSP/1.0\r\n"
                       "CSeq: 2\r\n"
                       "%.*s\r\n"
                       "\r\n",
                       s_url.mb_str().data(),
                       ( int )( p_c_session_end - p_c_session_start ),
                       p_c_session_start
                     ) ;
   /*--( Send the command )------------------------------------------------*/
   socket.Write( tb_c_buffer, i_length ) ;
   if( socket.Error() || ( int )socket.LastWriteCount() != i_length )
   {  return( false ) ; }
   /*--( Clean the rest )--------------------------------------------------*/
   socket.Read( tb_c_buffer, sizeof( tb_c_buffer ) ) ;
   if( socket.Error() ) { return( false ) ; }

   /*--( If we are here this means we have a positive result )-------------*/
   return( true ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CWinFbtvoOk::reset_gui_search()
{
   /*----------------------------------------------------------------------*/
   m_bt_start->Enable() ;
   m_ga_progress->SetValue( 0 ) ;
   m_pl->SetFocus() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CWinFbtvoOk::start_search()
{
   /*--( The declaration block is here because of the "goto" )-------------*/
   const CPlaylist *pl_fbtvo = wxGetApp().get_pk_pl( PK_FBTVO ) ;
   wxSocketClient socket ;
   mtf::t_vec_string vs_found ;
   mtf::t_vec_string vs_add ;
   wxURI uri ;
   wxURI uri_prev ;
   int i_nb_same = 0 ;

   /*----------------------------------------------------------------------*/
   const size_t co_sz_nb_option
          = ( pl_fbtvo == nullptr ? 0 : pl_fbtvo->get_vs_channel().size() ) ;
   /*----------------------------------------------------------------------*/
   if( co_sz_nb_option == 0 )
   {  msg_error( _( "No \"en option\" channel available" ) ) ;
      return ;
   }
   /*----------------------------------------------------------------------*/
   m_bt_start->Disable() ;
   m_ga_progress->SetRange( co_sz_nb_option - 1 ) ;
   msg_info( _( "Searching ..." ) ) ;
   /*----------------------------------------------------------------------*/
   for( size_t sz_pos = 0 ; sz_pos < co_sz_nb_option ; ++sz_pos )
   {
      /*-------------------------------------------------------------------*/
      CChannel *p_channel = pl_fbtvo->get_channel( sz_pos ) ;
      const str_stream *p_stream ;
      if(    p_channel != nullptr
          && ( p_stream = p_channel->get_stream() ) != nullptr
        )
      {
         /*--( Extract server to create the socket )-----------------------*/
         uri.Create( p_stream->m_s_url ) ;
         /*--( The socket is recreated only when needed )------------------*/
         if(    uri.GetScheme() != uri_prev.GetScheme()
             || uri.GetServer() != uri_prev.GetServer()
             || uri.GetPort()   != uri_prev.GetPort()
           )
         {  /*-------------------------------------------------------------*/
            if( !st_create_freebox_rtsp_socket( uri, socket ) )
            {  msg_error( _( "Connection problem with the Freebox" ) ) ;
               reset_gui_search() ;
               return ;
            }
            /*-------------------------------------------------------------*/
            uri_prev = uri ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
         if( st_check_rtsp_stream( socket, p_stream->m_s_url ) )
         {
            /*-------------------------------------------------------------*/
            vs_found.emplace_back( p_channel->get_name() ) ;
            if( m_pl->get_channel_item( p_channel->get_name() ) >= 0 )
            {  ++i_nb_same ; }
            else
            {  vs_add.emplace_back( p_channel->get_name() ) ; }
            /*-------------------------------------------------------------*/
            msg_info(   wxString::Format( "%d/%d ",
                                       static_cast< int >( vs_found.size() ),
                                       static_cast< int >( co_sz_nb_option )
                                        )
                      + wxPLURAL( "channel found", "channels found",
                                  vs_found.size()
                                )
                    ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      m_ga_progress->SetValue( sz_pos ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   int i_nb_add = 0 ;
   int i_nb_del = 0 ;
   wxString s_msg_add ;
   wxString s_msg_del ;
   /*----------------------------------------------------------------------*/
   i_nb_add = static_cast< int >( vs_add.size() ) ;
   if( m_pl->get_nb_item() > 0 )
   {  i_nb_del = m_pl->get_nb_item() - i_nb_same ; }
   /*----------------------------------------------------------------------*/
   if( i_nb_add != 0 )
   {  s_msg_add.Printf( wxPLURAL( "%d channel added",
                                  "%d channels added", i_nb_add
                                ),
                        i_nb_add
                      ) ;
      if( i_nb_del != 0 ) { s_msg_add += ", " ; }
   }
   /*----------------------------------------------------------------------*/
   if( i_nb_del != 0 )
   {  s_msg_del.Printf( wxPLURAL( "%d channel deleted",
                                  "%d channels deleted", i_nb_del
                                ),
                        i_nb_del
                      ) ;
   }
   /*----------------------------------------------------------------------*/
   if( s_msg_add.empty() && s_msg_del.empty() )
   {  msg_info( _( "No change found" ) ) ; }
   else
   {  msg_info( s_msg_add + s_msg_del ) ; }

   /*--( Finally add the channels found )----------------------------------*/
   m_pl->clear() ;
   if( !vs_found.empty() )
   {  /*-------------------------------------------------------------------*/
      m_pl->add_channel( vs_found ) ;
      /*--( The new channels are highlighted )-----------------------------*/
      if( !vs_add.empty() )
      {  /*----------------------------------------------------------------*/
         m_pl->set_item_hil( -1, false ) ;
         m_pl->set_item_hil( vs_add, true ) ;
         /*--( The focus is set to the 1st hil to avoid confusion )--------*/
         long l_item = m_pl->get_last_item_hil() ;
         if( l_item >= 0 )
         {  /*-------------------------------------------------------------*/
            m_pl->set_item_visible( l_item ) ;
            l_item = m_pl->get_first_item_hil() ;
            m_pl->set_item_focus( l_item ) ;
            m_pl->set_item_visible( l_item ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   reset_gui_search() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CWinFbtvoOk::msg_info( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetForegroundColour( *wxBLUE ) ;
   m_stx_msg->SetLabelText( s ) ;
   m_stx_msg->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CWinFbtvoOk::msg_error( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetForegroundColour( *wxRED ) ;
   m_stx_msg->SetLabelText( s ) ;
   m_stx_msg->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                      End of file CWinFbtvoOk.cpp                          !
+==========================================================================*/
